package anywheresoftware.b4a.RSA;

import anywheresoftware.b4a.BA.ShortName;

import anywheresoftware.b4a.RSA.utils.RSAKeyPair;
import anywheresoftware.b4a.RSA.utils.RSAPrivateKey;
import anywheresoftware.b4a.RSA.utils.RSAPublicKey;
import anywheresoftware.b4a.RSA.utils.RSAUtils;

@ShortName("Cipher")

/**
 * This class contains all the functions needed to generate the RSA Public and
 * Private Keys and stores them for using in encryption and decryption
 * 
 * @author pmartinez
 *
 */
public class Cipher {

	private RSAKeyPair cipherKey;
	private static int defaultBitLength = 1024;

	/**
	 * Create the Cipher element without cipher keys
	 */
	public Cipher() {
		super();
	}

	/**
	 * Create the cipher element with a public and a private key
	 * 
	 * @param cipherKey
	 *            The private and public key that will be used to
	 *            decrypt/encrypt
	 */
	public Cipher(RSAKeyPair cipherKey) {
		super();
		this.cipherKey = cipherKey;
	}

	/**
	 * Create the cipher element with the public key only
	 * 
	 * @param publicKey
	 *            The public key that will be used to encrypt
	 */
	public Cipher(RSAPublicKey publicKey) {
		super();
		this.cipherKey = new RSAKeyPair(publicKey, null);
	}

	/**
	 * Create the cipher element with the private key only
	 * 
	 * @param privateKey
	 *            The private key that will be used to encrypt
	 */
	public Cipher(RSAPrivateKey privateKey) {
		super();
		this.cipherKey = new RSAKeyPair(null, privateKey);
	}

	/**
	 * Generates the public and private keys
	 * 
	 * @param bitLength
	 *            Number of bits for generating the keys. Typically 1024.
	 * @return the public and private keys generated randomly for the bit length
	 *         introduced
	 */
	public RSAKeyPair generateKey(int bitLength) {
		cipherKey = RSAUtils.generateKey(bitLength);
		return cipherKey;
	}

	/**
	 * Generates the public and private keys with the default bit length (1024)
	 * 
	 * @return the public and private keys generated randomly for the default
	 *         bit length
	 */
	public RSAKeyPair generateKey() {
		cipherKey = RSAUtils.generateKey(defaultBitLength);
		return cipherKey;
	}

	/**
	 * @return the public and private keys of the cipher element
	 */
	public RSAKeyPair getCipherKey() {
		return cipherKey;
	}

	/**
	 * @param cipherKey
	 *            public and private keys to set in the cipher element
	 */
	public void setCipherKey(RSAKeyPair cipherKey) {
		this.cipherKey = cipherKey;
	}

	/**
	 * @param key
	 *            private key to set in the cipher element
	 */
	public void setPrivateKey(RSAPrivateKey key) {
		cipherKey.setPrivateKey(key);
	}

	/**
	 * @param key
	 *            public key to set in the cipher element
	 */
	public void setCipherKey(RSAPublicKey key) {
		cipherKey.setPublicKey(key);
	}

	/**
	 * @return the public key of the cipher element
	 */
	public RSAPublicKey getPublicKey() {
		return cipherKey.getPublicKey();
	}

	/**
	 * @return the private key of the cipher element
	 */
	public RSAPrivateKey getPrivateKey() {
		return cipherKey.getPrivateKey();
	}
}