package anywheresoftware.b4a.RSA.utils;

import anywheresoftware.b4a.BA.ShortName;

import java.math.BigInteger;
import java.util.Random;

@ShortName("RSAUtils")
/**
 * This utility class is used to generate the public and private keys, do the
 * encryption and decryption based on the RSA Algorithm
 * 
 * @author pmartinez
 *
 */
public class RSAUtils {

	/**
	 * Generate both, public and private key for a given bit length
	 * 
	 * @param bitLength
	 *            Number of bits that will be the strength for the keys (and the
	 *            processing complexity)
	 * @return Public and private keys
	 */
	public static RSAKeyPair generateKey(int bitLength) {
		Random r = new Random();
		BigInteger p = BigInteger.probablePrime(bitLength, r);
		BigInteger q = BigInteger.probablePrime(bitLength, r);
		BigInteger N = p.multiply(q);

		BigInteger phi = p.subtract(BigInteger.ONE).multiply(
				q.subtract(BigInteger.ONE));
		BigInteger e = BigInteger.probablePrime(bitLength / 2, r);

		while (phi.gcd(e).compareTo(BigInteger.ONE) > 0 && e.compareTo(phi) < 0) {
			e.add(BigInteger.ONE);
		}
		BigInteger d = e.modInverse(phi);

		return new RSAKeyPair(new RSAPublicKey(e, N), new RSAPrivateKey(d, N));
	}

	/**
	 * Encrypts the message with the given public key
	 * 
	 * @param message
	 *            the string to encrypt
	 * @param publicKey
	 *            the public key needed to encrypt
	 * @return the encrypted message
	 */
	public static byte[] encrypt(byte[] message, RSAPublicKey publicKey) {
		return (new BigInteger(message)).modPow(publicKey.getE(),
				publicKey.getN()).toByteArray();
	}

	/**
	 * Decrypts the message with the given private key
	 * 
	 * @param message
	 *            the string to decrypt
	 * @param privateKey
	 *            the private key needed to decrypt
	 * @return the decrypted message
	 */
	public static byte[] decrypt(byte[] message, RSAPrivateKey privateKey) {
		return (new BigInteger(message)).modPow(privateKey.getD(),
				privateKey.getN()).toByteArray();
	}
}
